gt_description = "Source Logic Tree v9.0.0"

src_correlations = {
    "dropped_group": "PUY",
    "correlations": [
        [
            {"group": "Hik", "tag": "Hik TL, N16.5, b0.95, C4, s0.42"},
            {"group": "PUY", "tag": "Puy 0.7, N4.6, b0.902, C4, s0.28"},
        ],
        [
            {"group": "Hik", "tag": "Hik TL, N21.5, b1.097, C4, s0.42"},
            {"group": "PUY", "tag": "Puy 0.7, N4.6, b0.902, C4, s0.28"},
        ],
        [
            {"group": "Hik", "tag": "Hik TL, N27.9, b1.241, C4, s0.42"},
            {"group": "PUY", "tag": "Puy 0.7, N4.6, b0.902, C4, s0.28"},
        ],
    ],
}

logic_tree_permutations = [
    [
        {
            "tag": "Final SLT for TAG Workshop - Hik fix, Cru fix, Puy fix",
            "weight": 1.0,
            "permute": [
                {
                    "group": "Hik",
                    "nrlz": 12,
                    "members": [
                        {
                            "tag": "Hik TL, N16.5, b0.95, C4, s0.42",
                            "weight": 0.576742825808845,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE2MTE=",
                            "bg_id": "RmlsZToxMzA3MzI=",
                        },
                        {
                            "tag": "Hik TL, N21.5, b1.097, C4, s0.42",
                            "weight": 0.347651982997022,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1OTk=",
                            "bg_id": "RmlsZToxMzA3Mzg=",
                        },
                        {
                            "tag": "Hik TL, N27.9, b1.241, C4, s0.42",
                            "weight": 0.0756051911941323,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1OTg=",
                            "bg_id": "RmlsZToxMzA3NDQ=",
                        },
                    ],
                },
                {
                    "group": "PUY",
                    "nrlz": 12,
                    "members": [
                        {
                            "tag": "Puy 0.7, N4.6, b0.902, C4, s0.28",
                            "weight": 1.0,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE2NTc=",
                            "bg_id": "RmlsZToxMzA3NTM=",
                        },
                    ],
                },
                {
                    "group": "CRU",
                    "nrlz": 21,
                    "members": [
                        {
                            "tag": "geodetic, TI, N2.7, b0.823 C4.2 s0.66",
                            "weight": 0.0168335471189857,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MDE=",
                            "bg_id": "RmlsZToxMzA3MDc=",
                        },
                        {
                            "tag": "geodetic, TI, N2.7, b0.823 C4.2 s1",
                            "weight": 0.0408928149352719,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0ODY=",
                            "bg_id": "RmlsZToxMzA3MTA=",
                        },
                        {
                            "tag": "geodetic, TI, N2.7, b0.823 C4.2 s1.41",
                            "weight": 0.0216771620919014,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MDI=",
                            "bg_id": "RmlsZToxMzA3MTM=",
                        },
                        {
                            "tag": "geodetic, TI, N3.4, b0.959 C4.2 s0.66",
                            "weight": 0.0282427120823285,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0ODI=",
                            "bg_id": "RmlsZToxMzA3MTY=",
                        },
                        {
                            "tag": "geodetic, TI, N3.4, b0.959 C4.2 s1",
                            "weight": 0.0686084751056566,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0ODM=",
                            "bg_id": "RmlsZToxMzA3MTk=",
                        },
                        {
                            "tag": "geodetic, TI, N3.4, b0.959 C4.2 s1.41",
                            "weight": 0.0363691528229985,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0OTA=",
                            "bg_id": "RmlsZToxMzA3MjI=",
                        },
                        {
                            "tag": "geodetic, TI, N4.6, b1.089 C4.2 s0.66",
                            "weight": 0.00792374079868575,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0OTE=",
                            "bg_id": "RmlsZToxMzA3MjU=",
                        },
                        {
                            "tag": "geodetic, TI, N4.6, b1.089 C4.2 s1",
                            "weight": 0.0192487099590715,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0OTY=",
                            "bg_id": "RmlsZToxMzA3Mjg=",
                        },
                        {
                            "tag": "geodetic, TI, N4.6, b1.089 C4.2 s1.41",
                            "weight": 0.0102036850851,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0OTQ=",
                            "bg_id": "RmlsZToxMzA3MzE=",
                        },
                        {
                            "tag": "geologic, TI, N2.7, b0.823 C4.2 s0.66",
                            "weight": 0.0236560148670262,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NzM=",
                            "bg_id": "RmlsZToxMzA3MDc=",
                        },
                        {
                            "tag": "geologic, TI, N2.7, b0.823 C4.2 s1",
                            "weight": 0.0574662625307477,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0ODE=",
                            "bg_id": "RmlsZToxMzA3MTA=",
                        },
                        {
                            "tag": "geologic, TI, N2.7, b0.823 C4.2 s1.41",
                            "weight": 0.0304626983900857,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0Njg=",
                            "bg_id": "RmlsZToxMzA3MTM=",
                        },
                        {
                            "tag": "geologic, TI, N3.4, b0.959 C4.2 s0.66",
                            "weight": 0.0271169544446554,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NjQ=",
                            "bg_id": "RmlsZToxMzA3MTY=",
                        },
                        {
                            "tag": "geologic, TI, N3.4, b0.959 C4.2 s1",
                            "weight": 0.0658737336745166,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NzY=",
                            "bg_id": "RmlsZToxMzA3MTk=",
                        },
                        {
                            "tag": "geologic, TI, N3.4, b0.959 C4.2 s1.41",
                            "weight": 0.0349194743556176,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0Njc=",
                            "bg_id": "RmlsZToxMzA3MjI=",
                        },
                        {
                            "tag": "geologic, TI, N4.6, b1.089 C4.2 s0.66",
                            "weight": 0.00222703068831837,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NjA=",
                            "bg_id": "RmlsZToxMzA3MjU=",
                        },
                        {
                            "tag": "geologic, TI, N4.6, b1.089 C4.2 s1",
                            "weight": 0.00541000379473566,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NzU=",
                            "bg_id": "RmlsZToxMzA3Mjg=",
                        },
                        {
                            "tag": "geologic, TI, N4.6, b1.089 C4.2 s1.41",
                            "weight": 0.00286782725429677,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE0NjM=",
                            "bg_id": "RmlsZToxMzA3MzE=",
                        },
                        {
                            "tag": "geodetic, TD, N2.7, b0.823 C4.2 s0.66",
                            "weight": 0.0168335471189857,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1Mzk=",
                            "bg_id": "RmlsZToxMzA3MDc=",
                        },
                        {
                            "tag": "geodetic, TD, N2.7, b0.823 C4.2 s1",
                            "weight": 0.0408928149352719,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1Mzc=",
                            "bg_id": "RmlsZToxMzA3MTA=",
                        },
                        {
                            "tag": "geodetic, TD, N2.7, b0.823 C4.2 s1.41",
                            "weight": 0.0216771620919014,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NTM=",
                            "bg_id": "RmlsZToxMzA3MTM=",
                        },
                        {
                            "tag": "geodetic, TD, N3.4, b0.959 C4.2 s0.66",
                            "weight": 0.0282427120823285,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NDQ=",
                            "bg_id": "RmlsZToxMzA3MTY=",
                        },
                        {
                            "tag": "geodetic, TD, N3.4, b0.959 C4.2 s1",
                            "weight": 0.0686084751056566,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NTk=",
                            "bg_id": "RmlsZToxMzA3MTk=",
                        },
                        {
                            "tag": "geodetic, TD, N3.4, b0.959 C4.2 s1.41",
                            "weight": 0.0363691528229985,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NTE=",
                            "bg_id": "RmlsZToxMzA3MjI=",
                        },
                        {
                            "tag": "geodetic, TD, N4.6, b1.089 C4.2 s0.66",
                            "weight": 0.00792374079868575,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NTg=",
                            "bg_id": "RmlsZToxMzA3MjU=",
                        },
                        {
                            "tag": "geodetic, TD, N4.6, b1.089 C4.2 s1",
                            "weight": 0.0192487099590715,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NDE=",
                            "bg_id": "RmlsZToxMzA3Mjg=",
                        },
                        {
                            "tag": "geodetic, TD, N4.6, b1.089 C4.2 s1.41",
                            "weight": 0.0102036850851,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1NjI=",
                            "bg_id": "RmlsZToxMzA3MzE=",
                        },
                        {
                            "tag": "geologic, TD, N2.7, b0.823 C4.2 s0.66",
                            "weight": 0.0236560148670263,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MjI=",
                            "bg_id": "RmlsZToxMzA3MDc=",
                        },
                        {
                            "tag": "geologic, TD, N2.7, b0.823 C4.2 s1",
                            "weight": 0.0574662625307477,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MjM=",
                            "bg_id": "RmlsZToxMzA3MTA=",
                        },
                        {
                            "tag": "geologic, TD, N2.7, b0.823 C4.2 s1.41",
                            "weight": 0.0304626983900857,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MjA=",
                            "bg_id": "RmlsZToxMzA3MTM=",
                        },
                        {
                            "tag": "geologic, TD, N3.4, b0.959 C4.2 s0.66",
                            "weight": 0.0271169544446554,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MTY=",
                            "bg_id": "RmlsZToxMzA3MTY=",
                        },
                        {
                            "tag": "geologic, TD, N3.4, b0.959 C4.2 s1",
                            "weight": 0.0658737336745166,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1Mjk=",
                            "bg_id": "RmlsZToxMzA3MTk=",
                        },
                        {
                            "tag": "geologic, TD, N3.4, b0.959 C4.2 s1.41",
                            "weight": 0.0349194743556176,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MTU=",
                            "bg_id": "RmlsZToxMzA3MjI=",
                        },
                        {
                            "tag": "geologic, TD, N4.6, b1.089 C4.2 s0.66",
                            "weight": 0.00222703068831837,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MzM=",
                            "bg_id": "RmlsZToxMzA3MjU=",
                        },
                        {
                            "tag": "geologic, TD, N4.6, b1.089 C4.2 s1",
                            "weight": 0.00541000379473566,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MjY=",
                            "bg_id": "RmlsZToxMzA3Mjg=",
                        },
                        {
                            "tag": "geologic, TD, N4.6, b1.089 C4.2 s1.41",
                            "weight": 0.00286782725429677,
                            "inv_id": "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEyOTE1MzE=",
                            "bg_id": "RmlsZToxMzA3MzE=",
                        },
                    ],
                },
                {
                    "group": "SLAB",
                    "nrlz": 12,
                    "members": [
                        {"tag": "slab-uniform-1depth-rates", "weight": 1.0, "inv_id": "", "bg_id": "RmlsZToxMjEwMzM="}
                    ],
                },
            ],
        }
    ]
]



import json
import warnings
from pathlib import Path
from typing import TYPE_CHECKING

from nzshm_model.psha_adapter import NrmlDocument, OpenquakeSimplePshaAdapter
from nzshm_model.source_logic_tree import SourceLogicTree, SourceLogicTreeV1

if TYPE_CHECKING:
    from nzshm_model.psha_adapter.openquake.logic_tree import LogicTree


RESOURCES_PATH = Path(__file__).parent.parent / "resources"
SLT_SOURCE_PATH = RESOURCES_PATH / "SRM_JSON"
GMM_SOURCE_PATH = RESOURCES_PATH / "GMM_LTs"


class NshmModel:
    def __init__(self, version, title, slt_json, gmm_xml, slt_config):
        self.version = version
        self.title = title
        self.slt_config = slt_config

        self._slt_json = SLT_SOURCE_PATH / slt_json
        self._gmm_xml = GMM_SOURCE_PATH / gmm_xml
        assert self._slt_json.exists()
        assert self._gmm_xml.exists()

    @property
    def _data(self):
        with open(self._slt_json, 'r') as jsonfile:
            data = json.load(jsonfile)
        return data

    def source_logic_tree(self) -> "SourceLogicTree":
        data = self._data
        ltv = data.get("logic_tree_version")
        if ltv is None:  # original json is unversioned
            return SourceLogicTree.from_source_logic_tree(SourceLogicTreeV1.from_dict(data))
        raise ValueError("Unsupported logic_tree_version.")

    def source_logic_tree_nrml(self) -> "LogicTree":
        warnings.warn("use NshmModel.source_logic_tree().psha_adapter().config() instead", DeprecationWarning)
        slt = self.source_logic_tree()
        return slt.psha_adapter(provider=OpenquakeSimplePshaAdapter).config()

    def gmm_logic_tree(self) -> "LogicTree":
        doc = NrmlDocument.from_xml_file(self._gmm_xml)
        return doc.logic_trees[0]



"""
Classes for deserialising NRML XML hazard into python dataclasses.

Should work for both GMM models and for Source Rate models.

Caveats

 - The outdated nrml schema NRML 0.4 is here:
   https://github.com/gem/oq-nrmllib/blob/master/openquake/nrmllib/schema/hazard/logic_tree.xsd

 - We can't find the NRML 0.5 schema

 - GEM do not enforce XSD schema validation. See https://groups.google.com/g/openquake-users/c/3BO_20hCsgg

 - This example source_model_logic_tree.xml shows an non-compliant source logic tree XML file that
   passes the openquake test suite:

    (https://github.com/gem/oq-engine/blob/6926a784f6026bef206a98cb4410be6c3e3c9273/
    openquake/qa_tests_data/logictree/case_01/source_model_logic_tree.xml)

NB: runzi.execute.openquake.util.oq_build_sources.py module contains code that
write source XML on the fly, using SLT python modules as inputs.

"""

from dataclasses import dataclass, field
from functools import lru_cache
from pathlib import Path, PurePath
from typing import TYPE_CHECKING, Any, Iterator, List, Union

from lxml import objectify

if TYPE_CHECKING:
    from nzshm_model.source_logic_tree.version2 import logic_tree as slt

from .uncertainty_models import (
    GenericUncertaintyModel,
    GroundMotionUncertaintyModel,
    NshmSourceUncertaintyModel,
    SourcesUncertaintyModel,
    _strip_whitespace,
)

NRML_NS = None
VALID_NRML_NS = ["http://openquake.org/xmlns/nrml/0.4", "http://openquake.org/xmlns/nrml/0.5"]


def get_nrml_namespace(element):
    namespaces = element.nsmap
    if namespaces[None] in VALID_NRML_NS:
        return namespaces[None]
    raise ValueError(f"the element {element} does not use a supported NRML namespace.")


@dataclass
class LogicTreeBranch:
    parent: "LogicTreeBranchSet"
    branchID: str
    uncertainty_models: List[Any] = field(default_factory=list)
    uncertainty_weight: float = 1.0

    @classmethod
    def from_parent_element(cls, ltbs: objectify.Element, parent: "LogicTreeBranchSet") -> Iterator["LogicTreeBranch"]:
        def uncertainty_models(
            ltb: objectify.Element, parent: "LogicTreeBranch", uncertainty_type
        ) -> Iterator[GenericUncertaintyModel]:
            for um in ltb.findall('nrml:uncertaintyModel', namespaces=NRML_NS):
                # here we allow client to override the class for different uncertainty model types,
                yield uncertainty_type.from_parent_element(um, parent)

        def uncertainty_weight(ltb: objectify.Element) -> float:
            uws = list(ltb.findall('nrml:uncertaintyWeight', namespaces=NRML_NS))
            if len(uws) == 1:
                return float(uws[0])
            raise ValueError("expecting exactly one uncertaintyWeight child, got {len(uws)}")

        for ltb in ltbs.iterchildren():
            _instance = LogicTreeBranch(
                parent=parent, branchID=ltb.get('branchID'), uncertainty_weight=uncertainty_weight(ltb)
            )
            _instance.uncertainty_models = list(uncertainty_models(ltb, _instance, parent.uncertainty_class()))
            yield _instance

    @classmethod
    def from_parent_slt(
        cls, slt_ltbs: "slt.FaultSystemLogicTree", parent: "LogicTreeBranchSet"
    ) -> Iterator["LogicTreeBranch"]:
        for ltb in slt_ltbs.branches:
            _instance = LogicTreeBranch(
                parent=parent,
                branchID=str(ltb.values),  # ltb.get('branchID'),
                uncertainty_weight=ltb.weight,  # LogicTreeBranch.uncertainty_weight(ltb)
            )
            _instance.uncertainty_models = list(NshmSourceUncertaintyModel.from_parent_slt(ltb, _instance))
            yield _instance

    def path(self) -> PurePath:
        return PurePath(self.parent.path(), _strip_whitespace(self.branchID))


@dataclass
class LogicTreeBranchSet:
    parent: "LogicTree"
    branchSetID: str  # assert ltbs.get('branchSetID') == "bs_crust"
    uncertaintyType: str  # assert ltbs.get('uncertaintyType') == "gmpeModel"
    applyToTectonicRegionType: str  # assert ltbs.get('applyToTectonicRegionType') == "Active Shallow Crust"

    branches: List['LogicTreeBranch'] = field(default_factory=list)

    @classmethod
    def from_parent_element(cls, logic_tree: objectify.Element, parent: "LogicTree") -> Iterator["LogicTreeBranchSet"]:
        # use of xpath here let's us ignore internediate elements such as logicTreeBranchingLevel in nrml/0.5
        for ltbs in logic_tree.xpath('//nrml:logicTreeBranchSet', namespaces=NRML_NS):
            _instance = LogicTreeBranchSet(
                parent=parent,
                branchSetID=ltbs.get('branchSetID'),
                uncertaintyType=ltbs.get('uncertaintyType'),
                applyToTectonicRegionType=ltbs.get('applyToTectonicRegionType'),
            )
            _instance.branches = list(LogicTreeBranch.from_parent_element(ltbs, _instance))
            yield (_instance)

    @classmethod
    def from_parent_slt(cls, slt: "slt.SourceLogicTree", parent: "LogicTree") -> Iterator["LogicTreeBranchSet"]:
        # assert slt_spec.fault_system_lts[0].short_name == "PUY"
        for ltbs in slt.fault_systems:
            _instance = LogicTreeBranchSet(
                parent=parent,
                branchSetID=ltbs.short_name,
                uncertaintyType="sourceModel",
                applyToTectonicRegionType="",
            )
            _instance.branches = list(LogicTreeBranch.from_parent_slt(ltbs, _instance))
            yield _instance

    def uncertainty_class(self):
        if self.uncertaintyType == "gmpeModel":
            return GroundMotionUncertaintyModel
        if self.uncertaintyType == "sourceModel":
            return SourcesUncertaintyModel
        return GenericUncertaintyModel

    def path(self) -> PurePath:
        return PurePath(self.parent.path(), _strip_whitespace(self.branchSetID))


@dataclass
class LogicTree:
    logicTreeID: str
    branch_sets: List['LogicTreeBranchSet'] = field(default_factory=list)

    @classmethod
    def from_parent_element(cls, root: objectify.Element) -> Iterator["LogicTree"]:
        for lt in root.xpath('/nrml:nrml/nrml:logicTree', namespaces=NRML_NS):
            _instance = LogicTree(logicTreeID=lt.get('logicTreeID'))
            _instance.branch_sets = list(LogicTreeBranchSet.from_parent_element(lt, _instance))
            yield _instance

    def path(self) -> PurePath:
        return PurePath(_strip_whitespace(self.logicTreeID))

    @classmethod
    def from_parent_slt(cls, slt: "slt.SourceLogicTree") -> "LogicTree":
        """
        build nrml instance from old-skool dataclasses instance.
        """
        _instance = LogicTree(logicTreeID=slt.version)
        _instance.branch_sets = list(LogicTreeBranchSet.from_parent_slt(slt, _instance))
        return _instance


@dataclass
class NrmlDocument:
    logic_trees: List[LogicTree] = field(default_factory=list)

    @classmethod
    @lru_cache
    def from_xml_file(cls, filepath: Union[Path, str]):
        gmm_tree = objectify.parse(filepath)
        root = gmm_tree.getroot()

        global NRML_NS
        NRML_NS = {'nrml': get_nrml_namespace(root)}
        return NrmlDocument(logic_trees=list(LogicTree.from_parent_element(root)))

    @classmethod
    def from_model_slt(cls, slt) -> "NrmlDocument":
        return NrmlDocument(logic_trees=[LogicTree.from_parent_slt(slt)])



#! python test_logic_tree.py

from pathlib import Path, PurePath

import pytest

from nzshm_model.psha_adapter import NrmlDocument
from nzshm_model.psha_adapter.openquake.uncertainty_models import _strip_whitespace

FIXTURE_PATH = Path(__file__).parent / "fixtures"


def test_nrml_gmm_logic_tree():
    gmm_logic_tree_path = FIXTURE_PATH / "TEST_GMM_LT.xml"
    doc = NrmlDocument.from_xml_file(gmm_logic_tree_path)

    logic_trees = list(doc.logic_trees)
    assert len(logic_trees) == 1

    crustal_branch = logic_trees[0].branch_sets[0]
    assert crustal_branch.applyToTectonicRegionType == "Active Shallow Crust"
    assert crustal_branch.branchSetID == "bs_crust"

    assert len(crustal_branch.branches) == 1

    # branch uncertainty weight
    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_weight == pytest.approx(0.117000)

    # branch uncertainty models
    assert "Stafford2022" in doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].text
    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].gmpe_name == "[Stafford2022]"
    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].arguments[0] == "mu_branch = \"Upper\""


def test_nrml_gmm_logic_tree_reverse_tree():
    gmm_logic_tree_path = FIXTURE_PATH / "TEST_GMM_LT.xml"
    doc = NrmlDocument.from_xml_file(gmm_logic_tree_path)

    assert (
        doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].parent
        is doc.logic_trees[0].branch_sets[0].branches[0]
    )

    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].parent.branchID == "STF22_upper"


def test_nrml_gmm_logic_tree_paths():
    gmm_logic_tree_path = FIXTURE_PATH / "TEST_GMM_LT.xml"
    doc = NrmlDocument.from_xml_file(gmm_logic_tree_path)

    # LogicTreeBranch path
    assert doc.logic_trees[0].branch_sets[0].branches[0].path() == PurePath("lt1", "bs_crust", "STF22_upper")
    assert doc.logic_trees[0].branch_sets[1].branches[0].path() == PurePath("lt1", "bs_slab", "Kuehn2020SS_GLO_lower")

    # uncertainty model path
    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].path() == PurePath(
        "lt1",
        "bs_crust",
        "STF22_upper",
        '[Stafford2022]mu_branch="Upper"',
    )

    assert doc.logic_trees[0].branch_sets[1].branches[0].uncertainty_models[0].path() == PurePath(
        "lt1",
        "bs_slab",
        "Kuehn2020SS_GLO_lower",
        '[KuehnEtAl2020SSlab]region="GLO"' 'sigma_mu_epsilon=-1.28155',
    )


SRC_TEST_DATA = [
    pytest.param(
        "TEST_SRC_LT_example_1.xml",
        "Combined",
        "BS-NONCE1",
        "CR_3km",
        1.0,
        "SW52ZXJzaW9uU29sdXRpb246MTAwMTkx-ruptures.xml\t\n"
        "            SW52ZXJzaW9uU29sdXRpb246MTAwMTkx-ruptures_sections.xml",
        id='example_1',
    ),
    pytest.param(
        "TEST_SRC_LT_example_2.xml",
        "Combined",
        "BS-NONCE1",
        "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEwNjQ2NA==|RmlsZToxMDY1MzM=|"
        "SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEwNjQ5Mg==|RmlsZToxMDY1NTQ=|SW52ZXJzaW9uU29sdXRpb25Ocm1sOjEwNjUwNg==|"
        "RmlsZToxMDY1NTM=",
        0.0833333,
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2MzYw-ruptures.xml\t"
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2MzYw-ruptures_sections.xml\t"
        "Floor_AddoptiEEPAScomb-INT_hiktcrp_b1.10_N17.18.xml\t"
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2NDEw-ruptures.xml\t"
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2NDEw-ruptures_sections.xml\t"
        "Floor_AddoptiEEPAScomb-INT_puy_b0.85_N4.36.xml\t"
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2NDE4-ruptures_sections.xml\t"
        "U2NhbGVkSW52ZXJzaW9uU29sdXRpb246MTA2NDE4-ruptures.xml\tFloor_AddoptiEEPAScomb-CRU_plyadj_b0.85_N4.36.xml",
        id='example_2',
    ),
]


@pytest.mark.parametrize(
    "fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model", SRC_TEST_DATA
)
def test_nrml_srm_logic_tree(
    fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model
):

    srm_logic_tree_path = FIXTURE_PATH / fixture_file
    doc = NrmlDocument.from_xml_file(srm_logic_tree_path)

    # logic tree 0
    assert len(doc.logic_trees) == 1
    assert doc.logic_trees[0].logicTreeID == logic_tree_id

    # branch_set 0
    assert len(doc.logic_trees[0].branch_sets) == 1
    assert doc.logic_trees[0].branch_sets[0].branchSetID == branch_set_id

    # branch 0
    assert len(doc.logic_trees[0].branch_sets[0].branches) == 1
    assert doc.logic_trees[0].branch_sets[0].branches[0].branchID == branch_id

    # branch 0 uncertainty weight
    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_weight == pytest.approx(uncertainty_weight)

    # branch 0 uncertainty models
    assert uncertainty_model in doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].text


@pytest.mark.parametrize(
    "fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model", SRC_TEST_DATA
)
def test_nrml_srm_logic_tree_parents(
    fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model
):

    srm_logic_tree_path = FIXTURE_PATH / fixture_file
    doc = NrmlDocument.from_xml_file(srm_logic_tree_path)

    # parental reference
    assert (
        doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].parent
        is doc.logic_trees[0].branch_sets[0].branches[0]
    )

    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].parent.branchID == branch_id


@pytest.mark.parametrize(
    "fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model", SRC_TEST_DATA
)
def test_nrml_srm_logic_tree_path(
    fixture_file, logic_tree_id, branch_set_id, branch_id, uncertainty_weight, uncertainty_model
):

    srm_logic_tree_path = FIXTURE_PATH / fixture_file
    doc = NrmlDocument.from_xml_file(srm_logic_tree_path)

    assert doc.logic_trees[0].branch_sets[0].branches[0].path() == PurePath(logic_tree_id, branch_set_id, branch_id)

    assert doc.logic_trees[0].branch_sets[0].branches[0].uncertainty_models[0].path() == PurePath(
        _strip_whitespace(logic_tree_id),
        _strip_whitespace(branch_set_id),
        _strip_whitespace(branch_id),
        _strip_whitespace(uncertainty_model),
    )



#! nshm_v1_0_0.py

import nzshm_model.source_logic_tree.SLT_v8_gmm_v2_final as slt_config

from .model import NshmModel

model = NshmModel(  # NOQA F401
    version='NSHM_v1.0.0',
    title="Initial version",
    slt_json="nshm_v1.0.0.json",
    gmm_xml="NZ_NSHM_GMM_LT_final_EE_new_names.xml",
    slt_config=slt_config,
)



"""
test migration from slt v1 to v2
"""
import pytest

import nzshm_model
from nzshm_model.source_logic_tree import SourceLogicTree, SourceLogicTreeV1


@pytest.fixture(scope='module')
def slt_version_1():
    # get a published v1 logic tree
    data = nzshm_model.get_model_version('NSHM_v1.0.4')._data
    yield SourceLogicTreeV1.from_dict(data)


def test_migrate_NSHM_v1_0_4(slt_version_1):
    v2 = SourceLogicTree.from_source_logic_tree(slt_version_1)
    assert v2.version == slt_version_1.version


def test_migrate_NSHM_v1_0_4_fault_systems(slt_version_1):
    v2 = SourceLogicTree.from_source_logic_tree(slt_version_1)
    assert v2.fault_systems[0].short_name == slt_version_1.fault_systems[0].short_name


def test_migrate_NSHM_v1_0_4_branches(slt_version_1):
    v2 = SourceLogicTree.from_source_logic_tree(slt_version_1)
    assert len(v2.fault_systems[0].branches) == len(slt_version_1.fault_systems[0].branches)
    assert len(v2.fault_systems[0].branches) == len(slt_version_1.fault_system_lts[0].branches)  # old attributes
    assert v2.fault_systems[0].branches[0].weight == slt_version_1.fault_systems[0].branches[0].weight



#! python test_logic_tree.py

from pathlib import Path

import pytest

from nzshm_model.psha_adapter import OpenquakeSimplePshaAdapter
from nzshm_model.source_logic_tree.version2 import SourceLogicTree

FIXTURE_PATH = Path(__file__).parent.parent.parent / "fixtures"


@pytest.mark.skip("WIP")
def test_source_logic_tree_to_source_xml_basic():

    config = FIXTURE_PATH / 'source_logic_tree_sample_0.json'
    slt = SourceLogicTree.from_json(config)

    print(slt)
    target = FIXTURE_PATH  # noqa
    src_xml = slt.psha_adapter(provider=OpenquakeSimplePshaAdapter).write_config(Path('/tmp/DEMO'))  # noqa
    # assert 0



#! python test_logic_tree.py

# import dataclasses
# import itertools
from pathlib import Path

from nzshm_model.source_logic_tree.version2 import SourceLogicTree

# import pytest


def test_v2_source_tree_from_json_no_correlations():
    config = Path(__file__).parent / 'fixtures' / 'source_logic_tree_sample_0.json'
    slt = SourceLogicTree.from_json(config)
    print(slt)
    # slt_spec = slt.derive_spec()
    # print(slt_spec)
    assert slt.logic_tree_version == 2
    assert slt.version == 'SLT_v9p0p0'
    assert slt.title == 'a SRM logic tree'
    assert slt.fault_systems[0].short_name == 'PUY'
    assert slt.fault_systems[0].long_name == 'Puysegur'
    assert slt.fault_systems[0].branches[0].values[0].name == 'dm'
    assert slt.fault_systems[0].branches[0].values[0].value == '0.7'
    # , bN[0.902, 4.6], C4.0, s0.28]
    # InversionSource
    assert slt.fault_systems[0].branches[0].sources[0].type == 'inversion'
    assert slt.fault_systems[0].branches[0].sources[0].nrml_id == 'SW52ZXJzaW9uU29sdXRpb25Ocm1sOjExODcxNw=='
    assert slt.fault_systems[0].branches[0].sources[0].rupture_rate_scaling == 0.9



import unittest

from nzshm_model.source_logic_tree import BranchAttributeSpec, BranchAttributeValue, SourceLogicTreeCorrelation


class TestFlattenedSourceLogicTree(unittest.TestCase):
    def test_correlation_compare(self):

        bn = BranchAttributeSpec(name='bN', long_name='bN pair', value_options=[[1.0, 2.0], [0.9, 3.0]])
        c = BranchAttributeSpec(name='C', long_name='area-magnitude-scaling', value_options=[1.0, 2.0])

        primary = [
            BranchAttributeValue.from_branch_attribute(bn, bn.value_options[0]),
            BranchAttributeValue.from_branch_attribute(c, c.value_options[0]),
        ]
        secondary = [
            BranchAttributeValue.from_branch_attribute(bn, bn.value_options[1]),
            BranchAttributeValue.from_branch_attribute(c, c.value_options[1]),
        ]

        correlation = SourceLogicTreeCorrelation('A', 'B', primary, secondary)

        self.assertTrue(correlation.is_primary(primary))
        self.assertFalse(correlation.is_primary(secondary))
        self.assertFalse(correlation.is_secondary(primary))
        self.assertTrue(correlation.is_secondary(secondary))



import logging
import pathlib
import zipfile
from typing import Any, Dict, Generator, Union

from lxml import etree
from lxml.builder import ElementMaker

from nzshm_model.psha_adapter.openquake.logic_tree import NrmlDocument
from nzshm_model.psha_adapter.psha_adapter_interface import PshaAdapterInterface

try:
    from .toshi import API_KEY, API_URL, SourceSolution
except (ModuleNotFoundError, ImportError):
    print('Running without `toshi` options')

QUICK_TEST = False

log = logging.getLogger(__name__)


def fetch_toshi_source(file_id: str, destination: pathlib.Path) -> pathlib.Path:
    headers = {"x-api-key": API_KEY}
    api = SourceSolution(API_URL, None, None, with_schema_validation=False, headers=headers)
    assert destination.exists()
    file_detail = api.get_source(file_id)
    fname = pathlib.Path(destination) / file_detail['file_name']

    if not fname.exists():
        api.download_file(file_id, destination)
        assert fname.exists()
    else:
        log.info(f'skipping existing: {fname}')
    return fname


def rupt_set_from_meta(meta):
    for itm in meta:
        if itm['k'] == "rupture_set_file_id":
            return itm['v']


class OpenquakeSimplePshaAdapter(PshaAdapterInterface):
    """
    Openquake PSHA simple nrml support.
    """

    def __init__(self, source_logic_tree):
        self._source_logic_tree = source_logic_tree
        assert source_logic_tree.logic_tree_version == 2

    def build_sources_xml(self, source_map):
        """Build a source model for a set of LTBs with their source files."""
        E = ElementMaker(
            namespace="http://openquake.org/xmlns/nrml/0.5",
            nsmap={"gml": "http://www.opengis.net/gml", None: "http://openquake.org/xmlns/nrml/0.5"},
        )
        NRML = E.nrml
        LT = E.logicTree
        LTBS = E.logicTreeBranchSet
        LTBL = E.logicTreeBranchingLevel
        LTB = E.logicTreeBranch
        UM = E.uncertaintyModel
        UW = E.uncertaintyWeight

        """
        # Build from existing NRML logic_tree
        nrml_logic_tree = self.config()
        # print(nrml_logic_tree)
        ltbl = LTBL(branchingLevelID="1")
        for branch_set in nrml_logic_tree.branch_sets:
            ltbs = LTBS(uncertaintyType="sourceModel", branchSetID=branch_set.branchSetID)
            for branch in branch_set.branches:
                ltb = LTB(UW(str(branch.uncertainty_weight)), branchID=branch.branchID)
                for um in branch.uncertainty_models:
                    files = "\n"
                    for filepath in source_map.get(um.toshi_nrml_id, []):
                        files += f"\t\"{filepath}\"\n"
                    ltb.append(UM(files))
                ltbs.append(ltb)
            ltbl.append(ltbs)
        """

        # Build from the source_logic_tree
        ltbl = LTBL(branchingLevelID="1")
        for fs in self.source_logic_tree.fault_systems:
            ltbs = LTBS(uncertaintyType="sourceModel", branchSetID=fs.short_name)
            for branch in fs.branches:
                branch_name = str(branch.values)
                files = ""
                ltv = getattr(self.source_logic_tree, "logic_tree_version", 0)
                if ltv >= 2:
                    # new logic trees
                    for source in branch.sources:
                        for filepath in source_map.get(source.nrml_id):
                            if not filepath.suffix == '.xml':
                                continue
                            files += f"\t{filepath}\n"
                        ltb = LTB(UM(files), UW(str(branch.weight)), branchID=branch_name)
                # else:
                #     # old style logic tree
                #     if source_map.get(branch.onfault_nrml_id):
                #         for filepath in source_map.get(branch.onfault_nrml_id):
                #             if not filepath.suffix == '.xml':
                #                 continue
                #             files += f"\t'{filepath}'\n"
                #     if source_map.get(branch.distributed_nrml_id):
                #         for filepath in source_map.get(branch.distributed_nrml_id):
                #             files += f"\t'{filepath}'\n"
                #     ltb = LTB(UM(files), UW(str(branch.weight)), branchID=str(branch.values))
                ltbs.append(ltb)
            ltbl.append(ltbs)
        nrml = NRML(LT(ltbl, logicTreeID="Combined"))
        return etree.tostring(nrml, pretty_print=True).decode()

    def write_config(
        self,
        cache_folder: Union[pathlib.Path, str],
        target_folder: Union[pathlib.Path, str],
        source_map: Union[None, Dict[str, list[pathlib.Path]]] = None,
    ) -> pathlib.Path:
        destination = pathlib.Path(target_folder)
        assert destination.exists()
        assert destination.is_dir()

        source_map = source_map or self.unpack_resources(cache_folder, target_folder)
        xmlstr = self.build_sources_xml(source_map)

        target_file = pathlib.Path(destination, 'sources.xml')
        with open(target_file, 'w') as fout:
            fout.write(xmlstr)
        return target_file

    def unpack_resources(
        self, cache_folder: Union[pathlib.Path, str], target_folder: Union[pathlib.Path, str]
    ) -> Dict[str, list[pathlib.Path]]:
        target = pathlib.Path(target_folder)
        target.mkdir(parents=True, exist_ok=True)
        source_map = {}
        limit = 2
        count = 0
        for file_id, filepath, uncertainty_model in self.fetch_resources(cache_folder):
            count += 1
            destination = target / uncertainty_model.path().parent
            destination.mkdir(parents=True, exist_ok=True)
            zf = zipfile.ZipFile(filepath)
            source_paths = []
            for name in zf.namelist():
                if not pathlib.Path(destination, name).exists():
                    zf.extract(name, destination)
                else:
                    log.info(f"skip existing {name}")
                source_paths.append(pathlib.Path(destination, name).relative_to(target))
            source_map[uncertainty_model.toshi_nrml_id] = source_paths
            if count >= limit and QUICK_TEST:
                break

        return source_map
        # # rename the extracted files
        # for name in zf.namelist():
        #     # rename
        #     extracted = pathlib.Path(destination, name)
        #     assert extracted.exists()
        #     if file_prefix:
        #         prefixed = pathlib.Path(destination, f"{file_prefix}_{name}")
        #         extracted.rename(prefixed)

    def fetch_resources(
        self, cache_folder: Union[pathlib.Path, str]
    ) -> Generator[tuple[Any, pathlib.Path, Any], None, None]:
        destination = pathlib.Path(cache_folder)
        destination.mkdir(parents=True, exist_ok=True)
        nrml_logic_tree = self.config()
        for branch_set in nrml_logic_tree.branch_sets:
            for branch in branch_set.branches:
                for um in branch.uncertainty_models:
                    filepath = fetch_toshi_source(um.toshi_nrml_id, destination)
                    yield um.toshi_nrml_id, filepath, um

    def config(self):
        return NrmlDocument.from_model_slt(self._source_logic_tree).logic_trees[0]

    @property
    def source_logic_tree(self):
        return self._source_logic_tree



#! test_module_import
import pytest

import nzshm_model


class TestStructure:
    def test_current_version(self):
        assert nzshm_model.CURRENT_VERSION == "NSHM_v1.0.0"

    def test_available_versions(self):
        assert len(nzshm_model.versions) == 2

    @pytest.mark.parametrize("model, model_version", [('NSHM_v1.0.0', 'NSHM_v1.0.0'), ('NSHM_v1.0.4', 'NSHM_v1.0.4')])
    def test_version_config(self, model, model_version):
        mod = nzshm_model.get_model_version(model)

        assert mod.version == model_version

        assert mod.slt_config is not None
        assert mod.slt_config.logic_tree_permutations is not None

        # slt = mod.source_logic_tree()
        # assert slt.version == model_version

    def test_get_model_version_unknown(self):
        unknown = nzshm_model.get_model_version('XXX')
        assert unknown is None



from . import nshm_v1_0_0, nshm_v1_0_4

# Python package version is different than the NSHM MODEL version !!
__version__ = '0.5.2'

CURRENT_VERSION = "NSHM_v1.0.0"

versions = {
    "NSHM_v1.0.0": nshm_v1_0_0.model,
    "NSHM_v1.0.4": nshm_v1_0_4.model,
}


def get_model_version(version_id):
    return versions.get(version_id)
